require 'versionomy'
require 'stringex'
require 'rainbow'
require 'rainbow/ext/string'

module Capistrano
  module Faucet
    module Utilities

        def who
          identity = (`git config user.name` || `whoami`)
          identity.chomp.to_url
        end

        def tags
          `git tag`.split("\n").compact
        end

        def non_release_tags
          tags - releases 
        end

        def current_branch
          branches.select{|b| b =~ /^\*\s/}.first.gsub(/^\*\s/,"")
        end

        def branches
         `git branch --no-color`.split("\n")
        end

        def version_tag_prefix
          `git config gitflow.prefix.versiontag`.split("\n").first
        end

        def releases
          tags.select{|t| t =~ /^#{version_tag_prefix}(\d+)/}.collect{|version| Versionomy.parse(version) }.sort
        end

        def latest_release
          releases.reverse.first.to_s
        end

        def available_tags
          puts "Available Tags:".color :green
          puts "#{releases.sort.reverse.take(Capistrano::Faucet::Configuration.previous_releases.to_i).join("\n")}"
          puts "\n"
        end

        def available_releases
          puts "\nAvailable Releases:".color :green
          puts "#{releases.reverse.take(Capistrano::Faucet::Configuration.previous_releases.to_i).join("\n")}"
          puts "\n"
        end

        def banner

          <<-BANNER
\n Faucet for Gitflow
          BANNER
        end



          def deploy_from
            puts banner unless Capistrano::Faucet::Configuration.prude.to_i == 1
            if fetch(:stage) == :production
              available_releases
              set :from_destination, ask("\nRelease to deploy:".color(:yellow).bright, latest_release)
            else
              set :create_tag, ask("\nDo you want to tag deployment?".color(:yellow).bright, 'N')
              return next_tag if fetch(:create_tag).downcase =~ /^[Yy]$/
              available_tags
              set :from_destination, ask("\nBranch, tag or release to deploy:".color(:yellow).bright, current_branch)
            end
            return fetch(:from_destination)
          end

          def next_tag
            hwhen   = Date.today.to_s
            set :what, ask("\nWhat does this release introduce?".color(:yellow).bright, nil)
            new_staging_tag = "#{hwhen}-#{who}-#{fetch(:what)}"
            puts "Tagging current branch for deployment to staging as '#{new_staging_tag}'".color(:green)
            system "git tag -a -m 'tagging current code for deployment to staging' #{new_staging_tag}"
            return new_staging_tag
          end

          def using_git?
            fetch(:scm, :git).to_sym == :git
          end
    end
  end
end
